@@ -1,11 +1,12 @@
 #coding:utf-8
 import logging
+import time
 from functools import wraps
 
 import requests
 from requests.exceptions import HTTPError
 
-from cloudbench.api.exceptions import NoSuchObject, MultipleObjectsReturned, APIError
+from cloudbench.api.exceptions import NoSuchObject, MultipleObjectsReturned, APIError, DuplicateObject
 from cloudbench.api.factory import _get_by_url, _list_objects, _create_object
 from cloudbench.api.util import path_join, _normalize_api_path
 
@@ -13,19 +14,34 @@ from cloudbench.api.util import path_join, _normalize_api_path
 logger = logging.getLogger(__name__)
 
 
+def counter():
+    i = 0
+    while 1:
+        yield i
+        i += 1
+
+
 def api_wrapper(method):
     @wraps(method)
     def wrapper(self, *args, **kwargs):
-        try:
-            return method(self, *args, **kwargs)
-        except HTTPError as e:
-            response = None
-            if hasattr(e, "response"):
-                response = e.response
-                logging.error("An API call failed")
-                logging.error("%s %s", response.status_code, response.reason)
-                logging.error(e.response.text)
-            raise APIError(response)
+
+        for n_failures in counter():
+            try:
+                return method(self, *args, **kwargs)
+            except HTTPError as e:
+                response = None
+                if hasattr(e, "response"):
+                    response = e.response
+                    logger.error("An API call failed")
+                    logger.error("%s %s", response.status_code, response.reason)
+                    logger.error(e.response.text)
+
+                if n_failures >= self.retry_max:
+                    raise APIError(response)
+
+                logger.info("Retrying API call in %s seconds", self.retry_wait)
+                time.sleep(self.retry_wait)
+
     return wrapper
 
 
@@ -63,15 +79,29 @@ class ResourceHandler(object):
         try:
             return self.get(**filters)
         except NoSuchObject:
-            return self.create(**filters)
+            try:
+                return self.create(**filters)
+            except DuplicateObject:
+                return self.get(**filters)
+
+    @property
+    def retry_max(self):
+        return self.client.retry_max
+
+    @property
+    def retry_wait(self):
+        return self.client.retry_wait
 
 
 class Client(object):
     _api_path = "api"
     _api_version = "v1"
 
-    def __init__(self, host, auth=None):
+    def __init__(self, host, auth=None, retry_max=0, retry_wait=0):
         self.host = host
+        self.retry_max = retry_max
+        self.retry_wait = retry_wait
+
         self._session = requests.Session()
         self._session.headers = {"accept": "application/json"}
         self._session.auth = auth
